@@ -34,13 +34,18 @@
         (include-list prj t))
       (list)))
 
+(tm-define (main-project-entry)
+  (buffer-list-menu (list (project-get))))
+
 (tm-define (project-list-menu)
   (buffer-list-menu (project-file-list)))
 
 (menu-bind project-menu
   (if (== (project-get) (current-buffer))
-      (link preamble-menu)
-      ---)
+      (link preamble-menu))
+  (if (!= (project-get) (current-buffer))
+      (link main-project-entry))
+  ---    
   (link project-list-menu))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -48,16 +53,17 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind document-style-menu
-  ("No style" (set-no-style))
-  ---
   (link style-menu)
   ---
+  ("No style" (set-no-style))
+  ("Edit style" (edit-style-source))
   ("Other style" (interactive set-main-style))
   ---
   (group "Customizations")
   (with l (get-style-list)
     (for (pack (if (null? l) l (cdr l)))
       (-> (eval (upcase-first pack))
+          ("Edit package" (edit-package-source pack))
           ("Remove package" (remove-style-package pack)))))
   (-> "Add package"
       (link toggle-package-menu)
@@ -354,17 +360,8 @@
 (menu-bind document-short-font-menu
   ("Default" (init-default-font))
   ---
-  ("Concrete" (init-font "concrete" "concrete"))
   ("Roman" (init-font "roman" "roman"))
   ("Stix" (init-font "stix" "math-stix"))
-  ;;---
-  ;;("Avant Garde" (init-env "font" "avant-garde"))
-  ;;("Bookman" (init-env "font" "bookman"))
-  ;;("Courier" (init-env "font" "courier"))
-  ;;("Helvetica" (init-env "font" "helvetica"))
-  ;;("NC Schoolbook" (init-env "font" "new-century-schoolbook"))
-  ;;("Palatino" (init-env "font" "palatino"))
-  ;;("Times" (init-env "font" "times"))
   (if (or (font-exists-in-tt? "texgyrebonum-math")
           (font-exists-in-tt? "texgyrepagella-math")
           (font-exists-in-tt? "texgyreschola-math")
@@ -463,9 +460,7 @@
   (-> "Number of columns"
       ("Default" (init-default "par-columns"))
       ---
-      ("1" (init-env "par-columns" "1"))
-      ("2" (init-env "par-columns" "2"))
-      ("3" (init-env "par-columns" "3")))
+      (link document-columns-menu))
   (-> "Advanced"
       (-> "Space stretchability"
           ("Default" (init-default "par-flexibility"))
@@ -518,22 +513,7 @@
   ("Standard 4:3" (init-page-type "4:3"))
   ("Standard 5:4" (init-page-type "5:4")))
 
-(tm-menu (document-page-size-menu)
-  ("Default"
-   (init-default "page-type" "page-width" "page-height")
-   (notify-page-change))
-  ---
-  (if (not (style-has? "beamer-style"))
-      (group "Common formats")
-      (link document-common-page-size-menu))
-  (if (style-has? "beamer-style")
-      (group "Beamer formats")
-      (link document-beamer-page-size-menu))
-  ---
-  (group "Standard formats")
-  (if (style-has? "beamer-style")
-      (-> "Common"
-          (link document-common-page-size-menu)))
+(tm-menu (document-standard-page-formats)
   (-> "A series"
       ("A0" (init-page-type "a0"))
       ("A1" (init-page-type "a1"))
@@ -586,13 +566,43 @@
       ("Note" (init-page-type "note"))
       ("Quarto" (init-page-type "quarto"))
       ("Statement" (init-page-type "statement"))
-      ("Tabloid" (init-page-type "tabloid")))
-  (if (not (style-has? "beamer-style"))
-      (-> "Beamer"
-          (link document-beamer-page-size-menu)))
+      ("Tabloid" (init-page-type "tabloid"))))
+
+(tm-menu (document-page-size-menu)
+  ("Default" (default-page-type))
+  ---
+  (group "Common formats")
+  (link document-common-page-size-menu)
+  ---
+  (group "Standard formats")
+  (link document-standard-page-formats)
+  (-> "Beamer"
+      (link document-beamer-page-size-menu))
   ---
   ("Other" (interactive init-page-size)))
 
+(tm-menu (document-page-size-menu)
+  (:require (style-has? "beamer-style"))
+  ("Default" (default-page-type))
+  ---
+  (group "Beamer formats")
+  (link document-beamer-page-size-menu)
+  ---
+  (group "Standard formats")
+  (-> "Common"
+      (link document-common-page-size-menu))
+  (link document-standard-page-formats)
+  ---
+  ("Other" (interactive init-page-size)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Number of columns
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (document-columns-menu)
+  ("One column" (init-env "par-columns" "1"))
+  ("Two columns" (init-env "par-columns" "2")))
+  
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The Document -> Page menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -819,6 +829,9 @@
   (-> "Paragraph" (link document-paragraph-menu))
   (-> "Page" (link document-page-menu))
   (-> "Metadata" (link document-metadata-menu))
+  (-> "Bibliography"
+      (when (buffer-has-biblio? (current-buffer))
+	("Local entries" (open-biblio))))
   ---
   (-> "Magnification" (link document-magnification-menu))
   (-> "Colors" (link document-colors-menu))
@@ -850,6 +863,8 @@
   ("Paragraph" (open-document-paragraph-format))
   ("Page" (open-document-page-format))
   ("Metadata" (open-document-metadata))
+  (when (buffer-has-biblio? (current-buffer))
+    ("Bibliography" (open-biblio)))
   ;;("Colors" (open-document-colors))
   ---
   (-> "Magnification" (link document-magnification-menu))
@@ -886,10 +901,12 @@
     (-> (eval (upcase-first (car st)))
         (link style-menu)
         ---
+        ("Edit style" (edit-style-source))
         ("Other style" (interactive set-main-style)))
     (dynamic (focus-style-extra-menu t))
     (for (pack (list-filter (cdr st) (negate hidden-package?)))
       (-> (eval (upcase-first pack))
+          ("Edit package" (edit-package-source pack))
           ("Remove package" (remove-style-package pack)))))
   (-> "Add style package"
       (link add-package-menu)
@@ -901,6 +918,8 @@
         ((== s "1") "One column")
         ((== s "2") "Two columns")
         ((== s "3") "Three columns")
+        ((== s "4") "Four columns")
+        ((== s "5") "Five columns")
         (else (string-append s " columns"))))
 
 (tm-menu (focus-document-menu t)
@@ -915,12 +934,10 @@
       ("Portrait" (init-page-orientation "portrait"))
       ("Landscape" (init-page-orientation "landscape")))
   (-> (eval (number-columns-text (get-init "par-columns")))
-      ("One column" (init-env "par-columns" "1"))
-      ("Two columns" (init-env "par-columns" "2")))
+      (link document-columns-menu))
   (if (and (== (get-preference "experimental encryption") "on")
 	   (!= (get-init "encryption") ""))
-      (=> (balloon (icon "tm_lock_open.xpm") "Encryption")
-	  (link document-encryption-menu)))
+      (-> "Encryption" (link document-encryption-menu)))
   (-> (eval (upcase-first (get-init "language")))
       (link document-language-menu))
   (-> (eval (upcase-first (font-family-main (get-init "font"))))
@@ -947,10 +964,12 @@
       (=> (balloon (eval (upcase-first (car st))) "Document style")
           (link style-menu)
           ---
+          ("Edit style" (edit-style-source))
           ("Other style" (interactive set-main-style)))
       (dynamic (focus-style-extra-icons t))
       (for (pack (list-filter (cdr st) (negate hidden-package?)))
         (=> (eval pack)
+            ("Edit package" (edit-package-source pack))
             ("Remove package" (remove-style-package pack)))))
     (=> (balloon (icon "tm_add.xpm") "Add style package")
         (link add-package-menu)
@@ -985,8 +1004,7 @@
         ("Portrait" (init-page-orientation "portrait"))
         ("Landscape" (init-page-orientation "landscape"))
         ---
-        ("One column" (init-env "par-columns" "1"))
-        ("Two columns" (init-env "par-columns" "2"))
+        (link document-columns-menu)
         ---
 	(link page-rendering-menu))
     (if (and (== (get-preference "experimental encryption") "on")
