@@ -82,6 +82,10 @@
   (r)
   (select-from-cursor))
 
+(tm-define (kbd-select-if-active r)
+  (r)
+  (select-from-cursor-if-active))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Basic editing via the keyboard
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -145,14 +149,13 @@
                      "tab"))))
 
 (tm-define (kbd-variant t forwards?)
-  (:require (and (tree-in? t '(label reference pageref)) (cursor-inside? t)))
+  (:require (and (tree-in? t '(label reference pageref eqref))
+                 (cursor-inside? t)))
   (if (complete-try?) (noop)))
 
 (tm-define (bib-cite-context? t)
   (and (tree-in? t '(cite nocite cite-detail))
-       (cursor-inside? t)
-       (or (not (tree-is? t 'cite-detail))
-           (== (tree-index (tree-down t)) 0))))
+       (cursor-inside? t)))
 
 (tm-define (kbd-variant t forwards?)
   (:require (and (not (supports-db?)) (bib-cite-context? t)))
@@ -267,7 +270,7 @@
   (and (tree-compound? t) (tree-label-extension? (tree-label t))))
 
 (tm-define (focus-has-preferences? t)
-  (:require (tree-in? t '(reference pageref hlink locus ornament)))
+  (:require (tree-in? t '(reference pageref eqref hlink locus ornament)))
   #t)
 
 (tm-define (focus-can-search? t)
@@ -664,12 +667,12 @@
 	"ornament-sunny-color" "ornament-shadow-color"))
 
 (tm-define (standard-parameters l)
-  (:require (in? l '("reference" "pageref" "label" "tag")))
+  (:require (in? l '("reference" "pageref" "eqref" "label" "tag")))
   (list))
 
 (tm-define (search-parameters l)
   (:require (in? (if (string? l) l (symbol->string l))
-                 '("reference" "pageref" "hlink")))
+                 '("reference" "pageref" "eqref" "hlink")))
   (standard-parameters "locus"))
 
 (tm-define (parameter-choice-list l)
@@ -720,20 +723,26 @@
       (insert-go-to `(draw-over ,selection (graphics) "0cm") '(1 1)))))
 
 (tm-define (make-graphics-over)
-  (with g `(with "gr-mode" (tuple "hand-edit" "line") (graphics))
-    (if (selection-active-any?)
+  (if (selection-active-any?)
+      (with g `(with "gr-mode" (tuple "hand-edit" "line") (graphics))
         (with selection (selection-tree)
           (clipboard-cut "graphics background")
-          (insert-go-to `(draw-over ,selection ,g "2cm") '(1 2 1)))
-        (insert-go-to `(draw-over "" ,g "2cm") '(1 2 1)))))
+          (insert-go-to `(draw-over ,selection ,g "2cm") '(1 2 1))))
+      (with g `(with "gr-mode" (tuple "hand-edit" "line")
+                     "gr-geometry" (tuple "geometry" "4cm" "4cm" "center")
+                 (graphics))
+        (insert-go-to `(draw-over "" ,g "2cm") '(1 4 1)))))
 
 (tm-define (make-anim l)
   (with duration "1s"
-    (if (selection-active-any?)
-        (with selection (selection-tree)
+    (if (selection-active?)
+        (let* ((selection (selection-tree))
+	       (p (path-end selection (list))))
+	  (when (selection-active-large?)
+	    (set! selection `(par-block ,selection))
+	    (set! p (cons 0 p)))
           (clipboard-cut "graphics background")
-          (insert-go-to `(,l ,selection ,duration)
-                        (cons 0 (path-end selection (list)))))
+          (insert-go-to `(,l ,selection ,duration) (cons 0 p)))
         (insert-go-to `(,l "" ,duration) (list 0 0)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -808,15 +817,18 @@
     (insert-go-to (list 'float s pos (list 'document ""))
                   (list 2 0 0))))
 
+(define (any-float? t)
+  (tree-in? t '(float wide-float phantom-float)))
+
 (tm-define (insertion-positioning what flag)
   (:synopsis "Allow/disallow the position @what for innermost float.")
-  (and-with t (tree-innermost 'float #t)
+  (and-with t (tree-innermost any-float? #t)
     (let ((op (if flag string-union string-minus))
           (st (tree-ref t 1)))
       (tree-set! st (op (tree->string st) what)))))
 
 (define (test-insertion-positioning? what)
-  (and-with t (tree-innermost 'float #t)
+  (and-with t (tree-innermost any-float? #t)
     (with c (string-ref what 0)
       (char-in-string? c (tree->string (tree-ref t 1))))))
 
